# Print an optspec for argparse to handle cmd's options that are independent of any subcommand.
function __fish_note_global_optspecs
	string join \n h/help
end

function __fish_note_needs_command
	# Figure out if the current invocation already has a command.
	set -l cmd (commandline -opc)
	set -e cmd[1]
	argparse -s (__fish_note_global_optspecs) -- $cmd 2>/dev/null
	or return
	if set -q argv[1]
		# Also print the command, so this can be used to figure out what it is.
		echo $argv[1]
		return 1
	end
	return 0
end

function __fish_note_using_subcommand
	set -l cmd (__fish_note_needs_command)
	test -z "$cmd"
	and return 1
	contains -- $cmd[1] $argv
end

complete -c note -n "__fish_note_needs_command" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_needs_command" -f -a "edit" -d 'Edits a note'
complete -c note -n "__fish_note_needs_command" -f -a "e" -d 'Edits a note'
complete -c note -n "__fish_note_needs_command" -f -a "journal" -d 'Opens the daily journal'
complete -c note -n "__fish_note_needs_command" -f -a "j" -d 'Opens the daily journal'
complete -c note -n "__fish_note_needs_command" -f -a "todo" -d 'Opens the todo file'
complete -c note -n "__fish_note_needs_command" -f -a "t" -d 'Opens the todo file'
complete -c note -n "__fish_note_needs_command" -f -a "search" -d 'Search the content of the notes'
complete -c note -n "__fish_note_needs_command" -f -a "s" -d 'Search the content of the notes'
complete -c note -n "__fish_note_needs_command" -f -a "find" -d 'Search the name of the files'
complete -c note -n "__fish_note_needs_command" -f -a "f" -d 'Search the name of the files'
complete -c note -n "__fish_note_needs_command" -f -a "query" -d 'Full text search of all the notes'
complete -c note -n "__fish_note_needs_command" -f -a "q" -d 'Full text search of all the notes'
complete -c note -n "__fish_note_needs_command" -f -a "sync" -d 'Sync the notes using the configured sync command'
complete -c note -n "__fish_note_needs_command" -f -a "list" -d 'List the notes in $NOTE_PATH or the current directory'
complete -c note -n "__fish_note_needs_command" -f -a "ls" -d 'List the notes in $NOTE_PATH or the current directory'
complete -c note -n "__fish_note_needs_command" -f -a "utils" -d 'Utility functions like shell completions'
complete -c note -n "__fish_note_needs_command" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c note -n "__fish_note_using_subcommand edit" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand e" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand journal" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand j" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand todo" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand t" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand search" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand s" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand find" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand f" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand query" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand q" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand sync" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand list" -s d -l max-depth -r
complete -c note -n "__fish_note_using_subcommand list" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand ls" -s d -l max-depth -r
complete -c note -n "__fish_note_using_subcommand ls" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand utils; and not __fish_seen_subcommand_from completion manpages help" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand utils; and not __fish_seen_subcommand_from completion manpages help" -f -a "completion" -d 'Generates shell completions for the given shell'
complete -c note -n "__fish_note_using_subcommand utils; and not __fish_seen_subcommand_from completion manpages help" -f -a "manpages" -d 'Generates the Man pages for the program'
complete -c note -n "__fish_note_using_subcommand utils; and not __fish_seen_subcommand_from completion manpages help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c note -n "__fish_note_using_subcommand utils; and __fish_seen_subcommand_from completion" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand utils; and __fish_seen_subcommand_from manpages" -s h -l help -d 'Print help'
complete -c note -n "__fish_note_using_subcommand utils; and __fish_seen_subcommand_from help" -f -a "completion" -d 'Generates shell completions for the given shell'
complete -c note -n "__fish_note_using_subcommand utils; and __fish_seen_subcommand_from help" -f -a "manpages" -d 'Generates the Man pages for the program'
complete -c note -n "__fish_note_using_subcommand utils; and __fish_seen_subcommand_from help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c note -n "__fish_note_using_subcommand help; and not __fish_seen_subcommand_from edit journal todo search find query sync list utils help" -f -a "edit" -d 'Edits a note'
complete -c note -n "__fish_note_using_subcommand help; and not __fish_seen_subcommand_from edit journal todo search find query sync list utils help" -f -a "journal" -d 'Opens the daily journal'
complete -c note -n "__fish_note_using_subcommand help; and not __fish_seen_subcommand_from edit journal todo search find query sync list utils help" -f -a "todo" -d 'Opens the todo file'
complete -c note -n "__fish_note_using_subcommand help; and not __fish_seen_subcommand_from edit journal todo search find query sync list utils help" -f -a "search" -d 'Search the content of the notes'
complete -c note -n "__fish_note_using_subcommand help; and not __fish_seen_subcommand_from edit journal todo search find query sync list utils help" -f -a "find" -d 'Search the name of the files'
complete -c note -n "__fish_note_using_subcommand help; and not __fish_seen_subcommand_from edit journal todo search find query sync list utils help" -f -a "query" -d 'Full text search of all the notes'
complete -c note -n "__fish_note_using_subcommand help; and not __fish_seen_subcommand_from edit journal todo search find query sync list utils help" -f -a "sync" -d 'Sync the notes using the configured sync command'
complete -c note -n "__fish_note_using_subcommand help; and not __fish_seen_subcommand_from edit journal todo search find query sync list utils help" -f -a "list" -d 'List the notes in $NOTE_PATH or the current directory'
complete -c note -n "__fish_note_using_subcommand help; and not __fish_seen_subcommand_from edit journal todo search find query sync list utils help" -f -a "utils" -d 'Utility functions like shell completions'
complete -c note -n "__fish_note_using_subcommand help; and not __fish_seen_subcommand_from edit journal todo search find query sync list utils help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c note -n "__fish_note_using_subcommand help; and __fish_seen_subcommand_from utils" -f -a "completion" -d 'Generates shell completions for the given shell'
complete -c note -n "__fish_note_using_subcommand help; and __fish_seen_subcommand_from utils" -f -a "manpages" -d 'Generates the Man pages for the program'

function __note_list_completion -d "Complete with the list of notes in a directory"
    set -l token (commandline -ct)

    if test $token != ''
        note list $token
    else
        note list
    end
end

complete -c note -n "__fish_seen_subcommand_from e edit" -k -f -a '(__note_list_completion)'
